import java.awt.*;
import java.net.*;

abstract class ImageArea extends Canvas
{ 
  Point      center=new Point(400,200);
  int        nx;
  int        ny;
  Image      table[][];
  Image     backBuffer;
  Graphics  bg;

  public Dimension fullSize()
  {
    return new Dimension(nx*200,ny*200);
  }
  
  abstract public URL urlBase();

  public ImageArea(int nx,int ny)
  {
    System.out.println("Size"+nx+":"+ny);
    this.nx=nx;
    this.ny=ny;
    table=new Image[nx+1][ny+1];
  }

  public Dimension preferredSize()
  {
    return fullSize();
  }

  public void setCenter(Point center)
  {
    this.center=center;
    repaint();
  }

  public Point getCenter()
  {
    return center;
  }

  public void reshape(int x, int y, int width, int height)
  {
    System.out.println("reshaping");
    System.out.println("\t"+width+":"+height);
    System.out.println("\t"+center);
    if(backBuffer!=null) backBuffer.flush();
    backBuffer = createImage(width,height);
    bg = backBuffer.getGraphics();
    super.reshape(x,y,width,height);
    repaint();
  }

  public void paint(Graphics g)
  {
    nx=fullSize().width/200;
    ny=fullSize().width/200;
    Dimension s    =size();
    Point     start=new Point(center.x-s.width/2,center.y-s.height/2);
    if(start.x<0) start.x=0;
    if(start.y<0) start.y=0;
    int xmin = start.x/200;
    int xoff = xmin*200 - start.x;
    int xmax = (start.x+s.width)/200;
    if((start.x+s.width)%200==0) xmax--;
    int ymin = start.y/200;
    int yoff = ymin*200 - start.y;
    int ymax = (start.y+s.height)/200;
    if((start.y+s.height)%200==0) ymax--;
    if(xmax>=nx ) xmax=nx-1;
    if(ymax>=ny ) ymax=ny-1;
    for(int x=xmin;x<=xmax;x++)
      for(int y=ymin;y<=ymax;y++)
	{
	  if(table[x][y]==null)
	    {
	      URL url;
	      try
		{
		  url= new URL(urlBase(),x+"-"+y+".gif");
		  table[x][y]=Toolkit.getDefaultToolkit().getImage(url);
		}
	      catch( MalformedURLException e)
		{
		  System.out.println(e);
		}
	    }
	  g.drawImage(table[x][y],(x-xmin)*200+xoff,
		      (y-ymin)*200+yoff,this);
	}
  }

  public void update(Graphics g)
  {
    paint(g);
  }

  public boolean imageUpdate(Image img,int infoflags,
			     int x,int y,int width,int height)
  {
    if((infoflags & (ALLBITS | FRAMEBITS)) != 0)
      {
	System.out.println("image redraw");
	repaint();
	return false;
      }
    return true;
  }
			     
}
